SETTING_FILENAME = 'filename'
SETTING_RECENT_FILES = 'recentFiles'
SETTING_WIN_SIZE = 'window/size'
SETTING_WIN_POSE = 'window/position'
SETTING_WIN_GEOMETRY = 'window/geometry'
SETTING_LINE_COLOR = 'line/color'
SETTING_FILL_COLOR = 'fill/color'
SETTING_ADVANCE_MODE = 'advanced'
SETTING_WIN_STATE = 'window/state'
SETTING_PROJECT_DIR = 'dir'
SETTING_PROJECT_FILE = 'projFile'
SETTING_PAINT_LABEL = 'paintlabel'
SETTING_LAST_OPEN_DIR = 'lastOpenDir'
SETTING_AUTO_SAVE = 'autosave'
SETTING_SINGLE_CLASS = 'singleclass'
FORMAT_PASCALVOC = 'PascalVOC'
FORMAT_YOLO = 'YOLO'
PROJECT_FILE_EXT = ".annt"
SETTING_DRAW_SQUARE = 'draw/square'
DEFAULT_ENCODING = 'utf-8'
MAX_RECENT = 7
XML_EXT = '.xml'
FILE_TREE_WIDTH_COL1 = 400
TXT_EXT = '.txt'
##Project files

PROJECT_VERIFIED_IMAGES = 'verifiedImgs'
PROJECT_NON_VERIFIED_IMAGES = 'nonVerifiedImgs'
PROJECT_CATEGORIES = 'categories'
PROJECT_ALL_IMAGES_NAMES = 'imageNames'
FILE_TREE_COLUMNS_NUMBER = 2
CONFIG_PATH = "seeds_faster/configs/seed/e2e_faster_rcnn_R_50_C4_1x_seed_strat2.yaml"

#20 colors to pick from

COLORS = ['#e6194b', '#3cb44b', '#ffe119', '#4363d8', '#f58231', 
    '#911eb4', '#46f0f0', '#f032e6', '#bcf60c', '#fabebe', '#008080',
    '#e6beff', '#9a6324', '#fffac8', '#800000', '#aaffc3', '#808000',
    '#ffd8b1', '#000075', '#808080', '#ffffff', '#000000']

##some parameters
IMAGE_DEFAULT_SCORE = 0.0
IMAGE_HUMAN_VERIFIED_SCORE = 1.0
