__version_info__ = ('0', '0', '32')
__version__ = '.'.join(__version_info__)
__authors__ = "Merey Ramazanova, Silvio Giancola"
__authors_username__ = "ramazam, giancos"
__author_email__ = "merey.ramazanova@kaust.edu.sa"
__github__ = 'https://github.com/meryusha/seeds_labeler/'